<?php

use Elementor\Modules\Usage\Module as Usage_Module;
use Elementor\Utils;

defined( 'ABSPATH' ) || die();

class Image_Generator_Post_Handler {

	public static function init() {

		if ( ! self::is_used_widget( 'wcf--image-generator' ) ) {
			return;
		}

		add_action( 'init', [ __CLASS__, 'init_post_type' ] );

		add_action( 'wp_footer', [ __CLASS__, 'render_image_generator_popup' ] );

		add_action( 'wp_ajax_sassly_get_render_images', [ __CLASS__, 'post_prepare_ajax' ] );
		add_action( 'wp_ajax_nopriv_sassly_get_render_images', [ __CLASS__, 'post_prepare_ajax' ] );
	}

	/**
	 * Post Tabs Ajax call
	 */
	public static function post_prepare_ajax() {

		if ( ! wp_verify_nonce( $_REQUEST['nonce'], 'sassly--addons-frontend' ) ) {
			exit( 'No naughty business please' );
		}

		parse_str( isset( $_POST['data_terms'] ) ? $_POST['data_terms'] : '', $form_data );

		$query = [
			'posts_per_page' => - 1,
			's'              => esc_attr( $form_data['s'] ),
			'post_type'      => 'wcf_image_generator'
		];

		$query = new WP_Query( $query );

		ob_start();

		self::display_post_template( $query );

		$html = ob_get_contents();  // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		ob_end_clean();

		wp_send_json(
			array(
				'html' => $html,
			)
		);

		die();
	}

	public static function is_used_widget( $widget ) {
		return array_key_exists( $widget, self::get_used_widgets() );
	}

	public static function get_used_widgets() {
		/** @var Usage_Module $usage_module */
		$usage_module = Usage_Module::instance();
		$usage_module->recalc_usage();

		$widgets_usage = [];
		foreach ( $usage_module->get_formatted_usage( 'raw' ) as $data ) {

			foreach ( $data['elements'] as $element => $count ) {
				if ( ! isset( $widgets_usage[ $element ] ) ) {
					$widgets_usage[ $element ] = 0;
				}

				$widgets_usage[ $element ] += $count;
			}
		}

		return $widgets_usage;
	}

	/**
	 * Initialize music post type
	 */
	public static function init_post_type() {

		$labels = [
			'name'           => __( 'Image Generator', 'sassly-essential' ),
			'singular_name'  => __( 'Image Generator', 'sassly-essential' ),
			'menu_name'      => __( 'Image Generator', 'sassly-essential' ),
			'name_admin_bar' => __( 'Image Generator', 'sassly-essential' ),
			'add_new'        => __( 'Add New', 'sassly-essential' ),
			'add_new_item'   => __( 'Add New ', 'sassly-essential' ),
			'new_item'       => __( 'New ', 'sassly-essential' ),
			'edit_item'      => __( 'Edit', 'sassly-essential' ),
			'view_item'      => __( 'View ', 'sassly-essential' ),
			'all_items'      => __( 'All ', 'sassly-essential' ),
			'search_items'   => __( 'Search ', 'sassly-essential' ),
		];

		register_post_type( 'wcf_image_generator',
			[
				'labels'            => $labels,
				'public'            => true,
				'has_archive'       => false,
				'show_in_nav_menus' => true,
				'menu_icon'         => 'dashicons-images-alt2',
				'supports'          => array( 'title', 'thumbnail', 'excerpt' )
			]
		);

		$taxonomy_labels = [
			'name'              => __( 'Category', 'sassly-essential' ),
			'singular_name'     => __( 'Category', 'sassly-essential' ),
			'search_items'      => __( 'Search Category', 'sassly-essential' ),
			'all_items'         => __( 'All Category', 'sassly-essential' ),
			'parent_item'       => __( 'Parent Category', 'sassly-essential' ),
			'parent_item_colon' => __( 'Parent Category', 'sassly-essential' ),
			'edit_item'         => __( 'Edit Category', 'sassly-essential' ),
			'update_item'       => __( 'Update Category', 'sassly-essential' ),
			'add_new_item'      => __( 'Add New Category', 'sassly-essential' ),
			'new_item_name'     => __( 'New Category Name', 'sassly-essential' ),
			'menu_name'         => __( 'Category', 'sassly-essential' ),
		];

		register_taxonomy( 'wcf_image_generator_cat', 'wcf_image_generator', [
			'label'             => __( 'Category', 'sassly-essential' ),
			"hierarchical"      => true,
			'labels'            => $taxonomy_labels,
			'show_admin_column' => true,
			'rewrite'           => array( 'slug' => 'wcf_image_generator/Category' )
		] );
	}

	public static function render_image_generator_popup() {
		?>
        <div class="wcf-image-generator-popup">
            <div class="wcf-image-generator-popup-wrapper">
                <div class="image-generator-post-wrapper">

                </div>
            </div>
        </div>
		<?php
	}

	public static function create_meta_for_post_type( $options, $prefix = '' ) {

		// Control core classes for avoid errors

		if ( ! class_exists( 'CSF' ) ) {
			return;
		}

		if ( empty( $options ) || empty( $options['meta_box'] ) ) {
			return;
		}

		// Set a unique slug-like ID
		if ( empty( $prefix ) ) {
			$prefix = 'wcf_post_options';
		}

		// Create a metabox
		CSF::createMetabox( $prefix, $options['meta_box'] );


		// Create a section
		if ( ! empty( $options['sections'] ) ) {

			foreach ( $options['sections'] as $section ) {
				CSF::createSection( $prefix, $section );
			}
		}

	}

	/**
	 * Render the music post widget output.
	 *
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	public static function display_post_template( $query = null ) {

		if ( is_null( $query ) ) {
			return;
		}

		if ( ! $query->found_posts ) {
			echo '<p class="no-post-found">' . esc_html__( 'No Post found' ) . '</p>';
		}

		while ( $query->have_posts() ) {
			$query->the_post();
			self::render_post();
		}
	}

	protected static function render_post() {
		if ( ! has_post_thumbnail( get_the_ID() ) ) {
			return;
		}
		?>
        <a href="<?php echo esc_url( get_the_post_thumbnail_url( get_the_ID(), 'full' ) ); ?>" class="image-generator-post">
			<?php self::render_thumbnail(); ?>
        </a>
		<?php
	}

	protected static function render_thumbnail() {
	    echo get_the_post_thumbnail( get_the_ID(), 'medium' );
	}

	protected static function render_title() {
		?>
        <h3 class="title">
            <a href="<?php echo esc_url( get_the_permalink() ); ?>">
				<?php the_title(); ?>
            </a>
        </h3>
		<?php
	}
}

Image_Generator_Post_Handler::init();
